package com.muyu.ruleEngine.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.annotation.Excel;
import com.muyu.ruleEngine.domain.req.EngineConfigQueryReq;
import com.muyu.ruleEngine.domain.req.EngineConfigSaveReq;
import com.muyu.ruleEngine.domain.req.EngineConfigEditReq;
import com.muyu.common.core.web.domain.BaseEntity;
import java.util.function.Supplier;
import java.util.Date;

/**
 * 引擎规则配置对象 engine_config
 *
 * @author DeKangLiu
 * @date 2024-05-02
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("engine_config")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "EngineConfig", description = "引擎规则配置")
public class EngineConfig extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** 编号 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "编号", value = "编号")
    private Long id;

    /** 版本编码 */
    @Excel(name = "版本编码")
    @ApiModelProperty(name = "版本编码", value = "版本编码", required = true)
    private String versionCode;

    /** 规则内容 */
    @Excel(name = "规则内容")
    @ApiModelProperty(name = "规则内容", value = "规则内容", required = true)
    private String ruleContent;

    /** 引擎维护编号 */
    @Excel(name = "引擎维护编号")
    @ApiModelProperty(name = "引擎维护编号", value = "引擎维护编号", required = true)
    private Long engineMaintenanceId;

    /**
    * 查询构造器
    */
    public static EngineConfig queryBuild( EngineConfigQueryReq engineConfigQueryReq){
        return EngineConfig.builder()
                .versionCode(engineConfigQueryReq.getVersionCode())
                .ruleContent(engineConfigQueryReq.getRuleContent())
                .engineMaintenanceId(engineConfigQueryReq.getEngineMaintenanceId())
                .build();
    }

    /**
    * 添加构造器
    */
    public static EngineConfig saveBuild(EngineConfigSaveReq engineConfigSaveReq,Supplier<String> createBy){
        return EngineConfig.builder()
                .versionCode(engineConfigSaveReq.getVersionCode())
                .ruleContent(engineConfigSaveReq.getRuleContent())
                .engineMaintenanceId(engineConfigSaveReq.getEngineMaintenanceId())
                .remark(engineConfigSaveReq.getRemark())
            .createBy(createBy.get())
            .createTime(new Date())
            .build();
    }

    /**
    * 修改构造器
    */
    public static EngineConfig editBuild(Long id, EngineConfigEditReq engineConfigEditReq, Supplier<String> updateBy){
        return EngineConfig.builder()
            .id(id)
                .versionCode(engineConfigEditReq.getVersionCode())
                .ruleContent(engineConfigEditReq.getRuleContent())
                .engineMaintenanceId(engineConfigEditReq.getEngineMaintenanceId())
                .remark(engineConfigEditReq.getRemark())
            .updateBy(updateBy.get())
            .updateTime(new Date())
            .build();
    }

}
