package com.muyu.ruleEngine.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.ruleEngine.service.EngineMaintenanceService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.ruleEngine.domain.EngineMaintenance;
import com.muyu.ruleEngine.domain.req.EngineMaintenanceQueryReq;
import com.muyu.ruleEngine.domain.req.EngineMaintenanceSaveReq;
import com.muyu.ruleEngine.domain.req.EngineMaintenanceEditReq;


/**
 * 引擎维护Controller
 *
 * @author DeKangLiu
 * @date 2024-05-02
 */
@Api(tags = "引擎维护")
@RestController
@RequestMapping("/maintenance")
public class EngineMaintenanceController extends BaseController {
    @Autowired
    private EngineMaintenanceService engineMaintenanceService;

    /**
     * 查询引擎维护列表
     */
    @ApiOperation("获取引擎维护列表")
    @RequiresPermissions("rule_engine:maintenance:list")
    @GetMapping("/list")
    public Result<TableDataInfo<EngineMaintenance>> list(EngineMaintenanceQueryReq engineMaintenanceQueryReq) {
        startPage();
        List<EngineMaintenance> list = engineMaintenanceService.list(EngineMaintenance.queryBuild(engineMaintenanceQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出引擎维护列表
     */
    @ApiOperation("导出引擎维护列表")
    @RequiresPermissions("rule_engine:maintenance:export")
    @Log(title = "引擎维护", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EngineMaintenance engineMaintenance) {
        List<EngineMaintenance> list = engineMaintenanceService.list(engineMaintenance);
        ExcelUtil<EngineMaintenance> util = new ExcelUtil<EngineMaintenance>(EngineMaintenance.class);
        util.exportExcel(response, list, "引擎维护数据");
    }

    /**
     * 获取引擎维护详细信息
     */
    @ApiOperation("获取引擎维护详细信息")
    @RequiresPermissions("rule_engine:maintenance:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<EngineMaintenance> getInfo(@PathVariable("id") Long id) {
        return Result.success(engineMaintenanceService.getById(id));
    }

    /**
     * 新增引擎维护
     */
    @RequiresPermissions("rule_engine:maintenance:add")
    @Log(title = "引擎维护", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增引擎维护")
    public Result<String> add(@RequestBody EngineMaintenanceSaveReq engineMaintenanceSaveReq) {
        return toAjax(engineMaintenanceService.saveEngineMaintenance(EngineMaintenance.saveBuild(engineMaintenanceSaveReq, SecurityUtils::getUsername)));
    }

    /**
     * 修改引擎维护
     */
    @RequiresPermissions("rule_engine:maintenance:edit")
    @Log(title = "引擎维护", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改引擎维护")
    public Result<String> edit(@PathVariable Long id, @RequestBody EngineMaintenanceEditReq engineMaintenanceEditReq) {
        return toAjax(engineMaintenanceService.updateById(EngineMaintenance.editBuild(id,engineMaintenanceEditReq, SecurityUtils::getUsername)));
    }

    /**
     * 删除引擎维护
     */
    @RequiresPermissions("rule_engine:maintenance:remove")
    @Log(title = "引擎维护", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除引擎维护")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(engineMaintenanceService.removeBatchEngineMaintenanceByIds(ids));
    }
}
