package com.muyu.ruleEngine.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.ruleEngine.domain.EngineConfig;
import com.muyu.ruleEngine.domain.model.TestData;
import com.muyu.ruleEngine.domain.resp.EngineConfigScopeResp;
import com.muyu.ruleEngine.dynamicLoad.DynamicLoader;
import com.muyu.ruleEngine.mapper.EngineConfigMapper;
import com.muyu.ruleEngine.service.EngineConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 引擎配置Service业务层处理
 *
 * @author DeKangLiu
 * @date 2024-05-02
 */
@Slf4j
@Service
public class EngineConfigServiceImpl extends ServiceImpl<EngineConfigMapper, EngineConfig> implements EngineConfigService {

    /**
     * 查询引擎配置作用域列表
     *
     * @return 引擎规则作用域集合
     */
    @Override
    public List<EngineConfigScopeResp> getScopeList() {
        List<EngineConfigScopeResp> list=new ArrayList<>();
        List<String> scopeName=List.of("TaskContextHolder","DataSetContextHolder","RecordContextHolder","DataModelContextHolder");
        try {
            for (String scope : scopeName) {
                String path="D:\\work\\2108A(3)\\cloud-server\\muyu-modules\\muyu-rule_engine\\muyu-rule_engine-server\\src\\main\\java\\com\\muyu\\ruleEngine\\scope\\"+scope+".java";
                String code = Files.readString(Paths.get(path));
                String type=null;
                if(scope.contains("Task")){
                    type="任务";
                }else if(scope.contains("DataSet")){
                    type="数据集";
                }else if(scope.contains("Record")){
                    type="资产记录";
                }else{
                    type="资产模型";
                }
                list.add(EngineConfigScopeResp.builder().type(type).name(scope).code(code).build());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return list;
    }

    /**
     * 查询引擎规则配置列表
     *
     * @param engineConfig 引擎规则配置
     * @return 引擎规则配置
     */
    @Override
    public List<EngineConfig> list(EngineConfig engineConfig) {
        LambdaQueryWrapper<EngineConfig> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(engineConfig.getVersionCode())){
            queryWrapper.eq(EngineConfig::getVersionCode, engineConfig.getVersionCode());
        }

        if (ObjUtils.notNull(engineConfig.getRuleContent())){
            queryWrapper.eq(EngineConfig::getRuleContent, engineConfig.getRuleContent());
        }

        if (ObjUtils.notNull(engineConfig.getEngineMaintenanceId())){
            queryWrapper.eq(EngineConfig::getEngineMaintenanceId, engineConfig.getEngineMaintenanceId());
        }





        return list(queryWrapper);
    }

    /**
     * 通过引擎作用域编号获取引擎配置作用域信息
     * @param id 引擎作用域编号
     * @return 引擎配置作用域信息
     */
    @Override
    public EngineConfigScopeResp getScopeInfoById(Integer id) {
        String scope=null;
        String type=null;
        if(id==0){
            type="测试模版";
            scope="TestClass.txt";
        }else if(id==1){
            type="任务";
            scope="TaskContextHolder.java";
        }else if(id==2){
            type="数据集";
            scope="DataSetContextHolder.java";
        }else if(id==3){
            type="资产记录";
            scope="RecordContextHolder.java";
        }else{
            type="资产模型";
            scope="DataModelContextHolder.java";
        }
        String path="D:\\work\\2108A(3)\\cloud-server\\muyu-modules\\muyu-rule_engine\\muyu-rule_engine-server\\src\\main\\java\\com\\muyu\\ruleEngine\\scope\\"+scope;
        String code = null;
        try {
            code = Files.readString(Paths.get(path));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return EngineConfigScopeResp.builder().type(type).name(scope).code(code).build();
    }

    /**
     * 规则测试
     * @param testData 规则配置版本编号
     */
    @Override
    public Object ruleTest(TestData testData) {
        Object invoke=null;
        try {
            String className="TestClass";
            final String SUFFIX = ".java";// 类名后面要跟的后缀
            EngineConfig config = this.getById(testData.getId());
            String content = config.getRuleContent().replaceAll("\r\n", "");
            // 对source进行编译生成class文件存放在Map中，这里用bytecode接收
            Map<String, byte[]> bytecode = DynamicLoader.compile(className + SUFFIX,content );

            // 加载class文件到虚拟机中，然后通过反射执行
            @SuppressWarnings("resource")
            DynamicLoader.MemoryClassLoader classLoader = new DynamicLoader.MemoryClassLoader(
                    bytecode);
            Class<?> clazz = classLoader.loadClass(className);

            // 调用ruleTest方法
            Method mainMethod = clazz.getDeclaredMethod("ruleTest", List.class);
            invoke = mainMethod.invoke(null, testData.getList());
        } catch (ClassNotFoundException | IllegalAccessException | NoSuchMethodException | InvocationTargetException e) {
            e.printStackTrace();
            throw new ServiceException("测试失败");
        }
        return invoke;
    }

}
