package com.muyu.ruleEngine.service.impl;

import java.util.List;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.ruleEngine.domain.EngineConfig;
import com.muyu.ruleEngine.domain.resp.EngineConfigScopeResp;
import com.muyu.ruleEngine.service.EngineConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.ruleEngine.mapper.EngineMaintenanceMapper;
import com.muyu.ruleEngine.domain.EngineMaintenance;
import com.muyu.ruleEngine.service.EngineMaintenanceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.transaction.annotation.Transactional;

/**
 * 引擎维护Service业务层处理
 *
 * @author DeKangLiu
 * @date 2024-05-02
 */
@Slf4j
@Service
public class EngineMaintenanceServiceImpl extends ServiceImpl<EngineMaintenanceMapper, EngineMaintenance>  implements EngineMaintenanceService {

    @Autowired
    private EngineConfigService engineConfigService;

    /**
     * 查询引擎维护列表
     *
     * @param engineMaintenance 引擎维护
     * @return 引擎维护
     */
    @Override
    public List<EngineMaintenance> list(EngineMaintenance engineMaintenance) {
        LambdaQueryWrapper<EngineMaintenance> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(engineMaintenance.getName())){
            queryWrapper.like(EngineMaintenance::getName, engineMaintenance.getName());
        }

        if (ObjUtils.notNull(engineMaintenance.getType())){
            queryWrapper.eq(EngineMaintenance::getType, engineMaintenance.getType());
        }

        if (ObjUtils.notNull(engineMaintenance.getScope())){
            queryWrapper.eq(EngineMaintenance::getScope, engineMaintenance.getScope());
        }

        if (ObjUtils.notNull(engineMaintenance.getEngineCode())){
            queryWrapper.eq(EngineMaintenance::getEngineCode, engineMaintenance.getEngineCode());
        }

        if (ObjUtils.notNull(engineMaintenance.getIsActivate())){
            queryWrapper.eq(EngineMaintenance::getIsActivate, engineMaintenance.getIsActivate());
        }

        if (ObjUtils.notNull(engineMaintenance.getStatus())){
            queryWrapper.eq(EngineMaintenance::getStatus, engineMaintenance.getStatus());
        }





        return list(queryWrapper);
    }

    /**
     * 新增引擎维护
     * @param engineMaintenance 引擎维护对象
     * @return 是否
     */
    @Override
    @Transactional
    public boolean saveEngineMaintenance(EngineMaintenance engineMaintenance) {
        boolean save = this.save(engineMaintenance);
        EngineConfigScopeResp scopeInfo = engineConfigService.getScopeInfoById(engineMaintenance.getScope());
        engineConfigService.save(EngineConfig.builder().versionCode("1.0").ruleContent(scopeInfo.getCode()).build());
        return save;
    }

    /**
     * 批量删除引擎维护
     * @param ids 引擎维护编号集合
     * @return 是否
     */
    @Override
    @Transactional
    public boolean removeBatchEngineMaintenanceByIds(List<Long> ids) {
        boolean removed = this.removeBatchByIds(ids);
        List<EngineConfig> engineConfigs = engineConfigService.list(new LambdaQueryWrapper<EngineConfig>()
                .in(EngineConfig::getEngineMaintenanceId, ids));
        if(!engineConfigs.isEmpty()){
            engineConfigService.removeBatchByIds(engineConfigs.stream().map(EngineConfig::getId).toList());
        }
        return removed;
    }


}
